from pprint import pp

from src.parsdantic.parser import parse

schema = {
    "$defs": {
        "Color": {
            "properties": {
                "id" : {"title": "ID", "type": "integer"},
                "r": {"title": "R", "type": "integer"},
                "g": {"title": "G", "type": "integer"},
                "b": {"title": "B", "type": "integer"},
            },
            "required": ["r", "g", "b"],
            "title": "Color",
            "type": "object",
        },
        "Hair": {
            "properties": {"id" : {"title": "ID", "type": "integer"}, "color": {"$ref": "#/$defs/Color"}},
            "required": ["color"],
            "title": "Hair",
            "type": "object",
        },
    },
    "properties": {
        "id" : {"title": "ID", "type": "integer"},
        "name": {"title": "Name", "type": "string"},
        "age": {"title": "Age", "type": "integer"},
        "hair": {"$ref": "#/$defs/Hair"},
    },
    "required": ["name", "age", "hair"],
    "title": "Person",
    "type": "object",
}

pydantic_model = parse(schema)

data = {
    "name": "John",
    "age": 30,
    "hair": {"color": {"r": 255, "g": 0, "b": 0}},
}

person = pydantic_model.model_validate(data)

pp(person.model_json_schema())


import pytest

from src.parsdantic.parser import parse


@pytest.mark.parametrize(
    "data, json_schema, expected",
    [
        (
            {"id": 1, "name": "name"},
            {"type": "object", "title" : "test", "properties": {"id": {"type": "integer"}, "name": {"type": "string"}}},
            True
        ),
        (
            {"id": 1, "name": "name", "obj": {"id": 2, "name": "name"}},
            {
                "type": "object",
                "title" : "test",
                "properties": {
                    "id": {"type": "integer"},
                    "name": {"type": "string"},
                    "obj": {
                        "title" : "test",
                        "type": "object",
                        "properties": {
                            "id": {"type": "integer"},
                            "name": {"type": "string"}
                        }
                    }
                }
            },
            True
        ),
        (
            {"id": 1, "name": "name", "obj": {"id": 2, "name": "name", "obj": {"id": 3, "name": "name"}}},
            {
                "type": "object",
                "title" : "test",
                "properties": {
                    "id": {"type": "integer"},
                    "name": {"type": "string"},
                    "obj": {
                        "title" : "test",
                        "type": "object",
                        "properties": {
                            "id": {"type": "integer"},
                            "name": {"type": "string"},
                            "obj": {
                                "title" : "test",
                                "type": "object",
                                "properties": {
                                    "id": {"type": "integer"},
                                    "name": {"type": "string"}
                                }
                            }
                        }
                    }
                }
            },
            True
        ),
    ]
)
def test_nested(data, json_schema, expected):
    try:
        pydantic_model = parse(json_schema)
        instance = pydantic_model(**data)
        pydantic_model.model_validate(instance)
        assert expected
    except Exception:
        assert not expected




import pytest

from src.parsdantic.parser import parse


@pytest.mark.parametrize(
    "data, json_schema, expected",
    [
        (
            {"id": 1, "names": ["name1", "name2"]},
            {"type": "object", "title" : "test", "properties": {"id": {"type": "integer"}, "names": {"type": "array", "items": {"type": "string"}}}},
            True
        ),
        (
            {"id": 1, "names": [["name1", "name2"], ["name3", "name4"]]},
            {"type": "object", "title" : "test", "properties": {"id": {"type": "integer"}, "names": {"type": "array", "items": {"type": "array", "items": {"type": "string"}}}}},
            True
        ),
    ]
)
def test_list(data, json_schema, expected):
    try:
        pydantic_model = parse(json_schema)
        instance = pydantic_model(**data)
        pydantic_model.model_validate(instance)
        assert expected
    except Exception:
        assert not expected




from typing import Any

import jsonref
from pydantic import BaseModel, create_model

Types = {"integer": int, "string": str, "boolean": bool}

def parse(obj: dict[str, Any]) -> type[BaseModel]:
    """Convert JSON Schema to Pydantic model

    Args:
        obj (dict[str, Any]): Model JSON Schema

    Returns:
        type[BaseModel]: Resulting Pydantic model
    """
    def _parse(title: str, obj: dict[str, dict[str, dict]], res={}) -> type[BaseModel]:
        """Convert JSON Schema to Pydantic model

        Args:
            title (str): Pydantic class name
            obj (dict[str, dict[str, dict]]): Properties JSON Schema
            res (dict, optional): _description_. Defaults to {}.

        Returns:
            type[BaseModel]: _description_
        """
        for k, v in obj.items():
            if v.get("properties"):
                res = res | {k : (_parse(v["title"], v["properties"]), None)}
            else:
                res = res | {k: (Types[v["type"]], None)}

        return create_model(title, **res)
    
    obj = jsonref.replace_refs(obj)
    return _parse(obj["title"], obj["properties"])
